==> mcp-1.0.1.tar.gz: mcp/src/Form/SettingsForm.php <==
<?php

declare(strict_types=1);

namespace Drupal\mcp\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Form\SubformState;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Configure Model Context Protocol settings for this site.
 */
class SettingsForm extends ConfigFormBase {

  /**
   * The MCP plugin manager.
   *
   * @var \Drupal\mcp\Plugin\McpPluginManager
   */
  protected $pluginManagerMcp;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $instance = parent::create($container);

    $instance->pluginManagerMcp = $container->get('plugin.manager.mcp');

    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'mcp_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames(): array {
    return ['mcp.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(
    array $form,
    FormStateInterface $form_state,
  ): array {
    $config = $this->config('mcp.settings');

    // General Settings container.
    $form['general'] = [
      '#type'  => 'details',
      '#title' => $this->t('General Settings'),
      '#open'  => TRUE,
    ];

    $form['general']['enable_auth'] = [
      '#type'          => 'checkbox',
      '#title'         => $this->t('Enable Auth'),
      '#description'   => $this->t(
        'Check to enable authentication for the MCP server. If disabled, the server will allow clients to connect with anonymous permissions.'
      ),
      '#default_value' => $config->get('enable_auth') ?? FALSE,
    ];

    $form['general']['auth_settings'] = [
      '#type'   => 'fieldset',
      '#title'  => $this->t('Authentication Settings'),
      '#states' => [
        'visible' => [
          ':input[name="enable_auth"]' => ['checked' => TRUE],
        ],
      ],
      '#tree'   => TRUE,
    ];

    $form['general']['auth_settings']['enable_token_auth'] = [
      '#type'          => 'checkbox',
      '#title'         => $this->t('Enable Token Auth'),
      '#description'   => $this->t(
        'Check to enable token-based authentication. This option grants administrative UID=1 permissions, so use with caution.'
      ),
      '#default_value' => $config->get('auth_settings.enable_token_auth') ??
      FALSE,
    ];

    $form['general']['auth_settings']['token_with_generation'] = [
      '#type'   => 'container',
      '#states' => [
        'visible' => [
          ':input[name="enable_token_auth"]' => ['checked' => TRUE],
        ],
      ],
    ];
    $form['general']['auth_settings']['token_with_generation']['token_key'] = [
      '#type'          => 'key_select',
      '#title'         => $this->t('Secret key'),
      '#default_value' => $config->get('auth_settings.token_key') ?? '',
    ];

    $form['general']['auth_settings']['enable_basic_auth'] = [
      '#type'          => 'checkbox',
      '#title'         => $this->t('Enable Basic Auth'),
      '#description'   => $this->t(
        'Check to enable Basic Authentication using a username and password. Authentication will enforce role-based permissions.'
      ),
      '#default_value' => $config->get('auth_settings.enable_basic_auth') ??
      FALSE,
    ];

    // Plugin settings wrapper.
    $form['plugins_wrapper'] = [
      '#type' => 'container',
      '#tree' => TRUE,
    ];

    // Vertical tabs to group plugin config forms.
    $form['plugins_wrapper']['tabs'] = [
      '#type'       => 'vertical_tabs',
      '#title'      => $this->t('Plugin Settings'),
      '#attributes' => ['style' => 'min-height: 300px;'],
    ];

    // Get each plugin definition from the manager.
    $availableInstances = $this->pluginManagerMcp->getAvailablePlugins(TRUE);
    usort($availableInstances, function ($a, $b) {
      $a_name = $a->getPluginDefinition()['name'] ?? $a->getPluginId();
      $b_name = $b->getPluginDefinition()['name'] ?? $b->getPluginId();
      if ($a_name instanceof TranslatableMarkup) {
        $a_name = $a_name->render();
      }
      if ($b_name instanceof TranslatableMarkup) {
        $b_name = $b_name->render();
      }

      return strnatcmp(
        $a_name, $b_name
      );
    });
    foreach ($availableInstances as $instance) {
      $plugin_id = $instance->getPluginId();
      $definition = $instance->getPluginDefinition();

      // Create a new details element for each plugin's sub-form.
      $form['plugins_wrapper'][$plugin_id] = [
        '#type'        => 'details',
        '#title'       => $definition['name'] ?? $instance->getPluginId(),
        '#description' => $definition['description'] ?? '',
        '#group'       => 'plugins_wrapper][tabs',
      ];

      // Build a subform for the plugin.
      $form['plugins_wrapper'][$plugin_id]['enabled'] = [
        '#type'          => 'checkbox',
        '#title'         => $this->t('Enable'),
        '#default_value' => $instance->getConfiguration()['enabled'] ?? TRUE,
      ];
      $form['plugins_wrapper'][$plugin_id]['config'] = [];
      $subformState = SubformState::createForSubform(
        $form['plugins_wrapper'][$plugin_id]['config'], $form, $form_state
      );

      // Let the plugin add its custom fields.
      $form['plugins_wrapper'][$plugin_id]['config']
        = $instance->buildConfigurationForm(
        $form['plugins_wrapper'][$plugin_id]['config'], $subformState
      );

      // If the plugin has no custom configuration options,
      // then don't show the plugin's details element at all.
      if (empty($form['plugins_wrapper'][$plugin_id]['config'])) {
        $form['plugins_wrapper'][$plugin_id]['config_message'] = [
          '#type'   => 'container',
          '#states' => [
            'visible' => [
              ':input[name="plugins_wrapper[' . $instance->getPluginId()
              . '][enabled]"]' => ['checked' => TRUE],
            ],
          ],
          'message' => [
            '#type'   => 'markup',
            '#markup' => new TranslatableMarkup(
              "No additional configuration options available."
            ),
            '#prefix' => '<p>',
            '#suffix' => '</p>',
          ],
        ];
      }
      else {
        // Wrap the plugin's config in a details element.
        $form['plugins_wrapper'][$plugin_id]['config'] = [
          '#type'   => 'details',
          '#open'   => TRUE,
          '#title'  => new TranslatableMarkup("Additional Configuration"),
          '#states' => [
            'visible' => [
              ':input[name="plugins_wrapper[' . $instance->getPluginId()
              . '][enabled]"]' => ['checked' => TRUE],
            ],
          ],
        ] + $form['plugins_wrapper'][$plugin_id]['config'];
      }
    }

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(
    array &$form,
    FormStateInterface $form_state,
  ): void {
    parent::validateForm($form, $form_state);

    if ($form_state->getValue('enable_auth')) {
      $auth_settings = $form_state->getValue('auth_settings');
      $enable_token_auth = !empty($auth_settings['enable_token_auth']);
      $enable_basic_auth = !empty($auth_settings['enable_basic_auth']);
      if (!$enable_token_auth && !$enable_basic_auth) {
        $form_state->setErrorByName(
          'auth_settings', $this->t(
          'At least one authentication method must be selected if Auth is enabled.'
        )
        );
      }
      if ($enable_token_auth) {
        $token_key = $auth_settings['token_with_generation']['token_key'] ?? '';
        if (empty($token_key)) {
          $form_state->setErrorByName(
            'token_with_generation][token_key', $this->t(
            'Secret key must be provided when token authentication is enabled.'
          )
          );
        }
      }
    }

    $plugins_values = $form_state->getValue('plugins_wrapper') ?? [];
    foreach ($plugins_values as $plugin_id => $plugin_values) {
      if ($plugin_id === 'tabs') {
        continue;
      }

      $instance = $this->pluginManagerMcp->createInstance($plugin_id);
      $subformState = SubformState::createForSubform(
        $form['plugins_wrapper'][$plugin_id],
        $form,
        $form_state
      );

      // Let the plugin do custom validation if needed.
      $instance->validateConfigurationForm(
        $form['plugins_wrapper'][$plugin_id], $subformState
      );
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(
    array &$form,
    FormStateInterface $form_state,
  ): void {
    // Load our config object for editing.
    $config = $this->config('mcp.settings');

    // Save general settings.
    $auth_settings = $form_state->getValue('auth_settings');
    $enable_auth = (bool) $form_state->getValue('enable_auth');
    $config->set('enable_auth', $enable_auth);
    $config->set(
      'auth_settings', $enable_auth ? [
        'enable_token_auth' => !empty($auth_settings['enable_token_auth'])
          ? $auth_settings['enable_token_auth'] : FALSE,
        'token_key'         => $auth_settings['token_with_generation']['token_key']
        ?? '',
        'enable_basic_auth' => !empty($auth_settings['enable_basic_auth'])
          ? $auth_settings['enable_basic_auth'] : FALSE,
      ] : []
    );

    // Save plugin settings.
    $plugins_values = $form_state->getValue('plugins_wrapper') ?? [];
    foreach ($plugins_values as $plugin_id => $plugin_values) {
      if ($plugin_id === 'tabs') {
        continue;
      }

      // Get the existing plugin instance with stored config:
      $instance = $this->pluginManagerMcp->createInstance($plugin_id);

      // Let it process its form submission.
      $subformState = SubformState::createForSubform(
        $form['plugins_wrapper'][$plugin_id],
        $form,
        $form_state
      );
      $instance->submitConfigurationForm(
        $form['plugins_wrapper'][$plugin_id],
        $subformState
      );

      // Now $instance->configuration is updated.
      // Write it back into our module config array.
      $config->set("plugins.$plugin_id", $instance->getConfiguration());
    }

    // Save everything to mcp.settings.
    $config->save();

    // Call parent submit to show success message, etc.
    parent::submitForm($form, $form_state);
  }

}

==> mcp-1.1.0.tar.gz: mcp/src/Form/SettingsForm.php <==
<?php

declare(strict_types=1);

namespace Drupal\mcp\Form;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Form\SubformState;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Configure Model Context Protocol settings for this site.
 */
class SettingsForm extends ConfigFormBase {

  /**
   * The MCP plugin manager.
   *
   * @var \Drupal\mcp\Plugin\McpPluginManager
   */
  protected $pluginManagerMcp;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected EntityTypeManagerInterface $entityTypeManager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $instance = parent::create($container);

    $instance->pluginManagerMcp = $container->get('plugin.manager.mcp');
    $instance->entityTypeManager = $container->get('entity_type.manager');

    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'mcp_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames(): array {
    return ['mcp.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(
    array $form,
    FormStateInterface $form_state,
  ): array {
    $config = $this->config('mcp.settings');

    // General Settings container.
    $form['general'] = [
      '#type'  => 'details',
      '#title' => $this->t('General Settings'),
      '#open'  => TRUE,
    ];

    $form['general']['enable_auth'] = [
      '#type'          => 'checkbox',
      '#title'         => $this->t('Enable Auth'),
      '#description'   => $this->t(
        'Check to enable authentication for the MCP server. If disabled, the server will allow clients to connect with anonymous permissions.'
      ),
      '#default_value' => $config->get('enable_auth') ?? FALSE,
    ];

    $form['general']['auth_settings'] = [
      '#type'   => 'fieldset',
      '#title'  => $this->t('Authentication Settings'),
      '#states' => [
        'visible' => [
          ':input[name="enable_auth"]' => ['checked' => TRUE],
        ],
      ],
      '#tree'   => TRUE,
    ];

    $form['general']['auth_settings']['enable_token_auth'] = [
      '#type'          => 'checkbox',
      '#title'         => $this->t('Enable Token Auth'),
      '#description'   => $this->t(
        'Check to enable token-based authentication. Token authentication allows access as a specific user.'
      ),
      '#default_value' => $config->get('auth_settings.enable_token_auth') ??
      FALSE,
    ];

    $form['general']['auth_settings']['token_with_generation'] = [
      '#type'   => 'container',
      '#states' => [
        'visible' => [
          ':input[name="enable_token_auth"]' => ['checked' => TRUE],
        ],
      ],
    ];
    $form['general']['auth_settings']['token_with_generation']['token_key'] = [
      '#type'          => 'key_select',
      '#title'         => $this->t('Secret key'),
      '#default_value' => $config->get('auth_settings.token_key') ?? '',
      '#states'        => [
        'visible' => [
          ':input[name="auth_settings[enable_token_auth]"]' => ['checked' => TRUE],
        ],
        'required' => [
          ':input[name="auth_settings[enable_token_auth]"]' => ['checked' => TRUE],
        ],
      ],
    ];

    $form['general']['auth_settings']['token_with_generation']['token_user'] = [
      '#type'          => 'entity_autocomplete',
      '#title'         => $this->t('Token User'),
      '#description'   => $this->t(
        'Select the user account that will be used for token authentication.'
      ),
      '#target_type'   => 'user',
      '#default_value' => $config->get('auth_settings.token_user') ?
      $this->entityTypeManager->getStorage('user')->load(
          $config->get('auth_settings.token_user')
      ) : NULL,
      '#states'        => [
        'visible' => [
          ':input[name="auth_settings[enable_token_auth]"]' => ['checked' => TRUE],
        ],
        'required' => [
          ':input[name="auth_settings[enable_token_auth]"]' => ['checked' => TRUE],
        ],
      ],
    ];

    $form['general']['auth_settings']['enable_basic_auth'] = [
      '#type'          => 'checkbox',
      '#title'         => $this->t('Enable Basic Auth'),
      '#description'   => $this->t(
        'Check to enable Basic Authentication using a username and password. Authentication will enforce role-based permissions.'
      ),
      '#default_value' => $config->get('auth_settings.enable_basic_auth') ??
      FALSE,
    ];

    // Plugin settings wrapper.
    $form['plugins_wrapper'] = [
      '#type' => 'container',
      '#tree' => TRUE,
    ];

    // Vertical tabs to group plugin config forms.
    $form['plugins_wrapper']['tabs'] = [
      '#type'       => 'vertical_tabs',
      '#title'      => $this->t('Plugin Settings'),
      '#attributes' => ['style' => 'min-height: 300px;'],
    ];

    // Get each plugin definition from the manager.
    $availableInstances = $this->pluginManagerMcp->getAvailablePlugins(TRUE);
    usort($availableInstances, function ($a, $b) {
      $a_name = $a->getPluginDefinition()['name'] ?? $a->getPluginId();
      $b_name = $b->getPluginDefinition()['name'] ?? $b->getPluginId();
      if ($a_name instanceof TranslatableMarkup) {
        $a_name = $a_name->render();
      }
      if ($b_name instanceof TranslatableMarkup) {
        $b_name = $b_name->render();
      }

      return strnatcmp(
        $a_name, $b_name
      );
    });
    foreach ($availableInstances as $instance) {
      $plugin_id = $instance->getPluginId();
      $definition = $instance->getPluginDefinition();

      // Create a new details element for each plugin's sub-form.
      $form['plugins_wrapper'][$plugin_id] = [
        '#type'        => 'details',
        '#title'       => $definition['name'] ?? $instance->getPluginId(),
        '#description' => $definition['description'] ?? '',
        '#group'       => 'plugins_wrapper][tabs',
      ];

      // Build a subform for the plugin.
      $form['plugins_wrapper'][$plugin_id]['enabled'] = [
        '#type'          => 'checkbox',
        '#title'         => $this->t('Enable'),
        '#default_value' => $instance->getConfiguration()['enabled'] ?? TRUE,
      ];
      $form['plugins_wrapper'][$plugin_id]['config'] = [];
      $subformState = SubformState::createForSubform(
        $form['plugins_wrapper'][$plugin_id]['config'], $form, $form_state
      );

      // Let the plugin add its custom fields.
      $form['plugins_wrapper'][$plugin_id]['config']
        = $instance->buildConfigurationForm(
        $form['plugins_wrapper'][$plugin_id]['config'], $subformState
      );

      // If the plugin has no custom configuration options,
      // then don't show the plugin's details element at all.
      if (empty($form['plugins_wrapper'][$plugin_id]['config'])) {
        $form['plugins_wrapper'][$plugin_id]['config_message'] = [
          '#type'   => 'container',
          '#states' => [
            'visible' => [
              ':input[name="plugins_wrapper[' . $instance->getPluginId()
              . '][enabled]"]' => ['checked' => TRUE],
            ],
          ],
          'message' => [
            '#type'   => 'markup',
            '#markup' => new TranslatableMarkup(
              "No additional configuration options available."
            ),
            '#prefix' => '<p>',
            '#suffix' => '</p>',
          ],
        ];
      }
      else {
        // Wrap the plugin's config in a details element.
        $form['plugins_wrapper'][$plugin_id]['config'] = [
          '#type'   => 'details',
          '#open'   => TRUE,
          '#title'  => new TranslatableMarkup("Additional Configuration"),
          '#states' => [
            'visible' => [
              ':input[name="plugins_wrapper[' . $instance->getPluginId()
              . '][enabled]"]' => ['checked' => TRUE],
            ],
          ],
        ] + $form['plugins_wrapper'][$plugin_id]['config'];
      }
    }

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(
    array &$form,
    FormStateInterface $form_state,
  ): void {
    parent::validateForm($form, $form_state);

    if ($form_state->getValue('enable_auth')) {
      $auth_settings = $form_state->getValue('auth_settings');
      $enable_token_auth = !empty($auth_settings['enable_token_auth']);
      $enable_basic_auth = !empty($auth_settings['enable_basic_auth']);
      if (!$enable_token_auth && !$enable_basic_auth) {
        $form_state->setErrorByName(
          'auth_settings', $this->t(
          'At least one authentication method must be selected if Auth is enabled.'
        )
        );
      }
      if ($enable_token_auth) {
        $token_key = $auth_settings['token_with_generation']['token_key'] ?? '';
        if (empty($token_key)) {
          $form_state->setErrorByName(
            'token_with_generation][token_key', $this->t(
            'Secret key must be provided when token authentication is enabled.'
          )
          );
        }

        // Validate token user selection.
        $token_user = $auth_settings['token_with_generation']['token_user'] ??
          '';
        if (empty($token_user)) {
          $form_state->setErrorByName(
            'token_with_generation][token_user', $this->t(
            'A user must be selected for token authentication.'
          )
          );
        }
      }
    }

    $plugins_values = $form_state->getValue('plugins_wrapper') ?? [];
    foreach ($plugins_values as $plugin_id => $plugin_values) {
      if ($plugin_id === 'tabs') {
        continue;
      }

      $instance = $this->pluginManagerMcp->createInstance($plugin_id);
      $subformState = SubformState::createForSubform(
        $form['plugins_wrapper'][$plugin_id],
        $form,
        $form_state
      );

      // Let the plugin do custom validation if needed.
      $instance->validateConfigurationForm(
        $form['plugins_wrapper'][$plugin_id], $subformState
      );
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(
    array &$form,
    FormStateInterface $form_state,
  ): void {
    // Load our config object for editing.
    $config = $this->config('mcp.settings');

    // Save general settings.
    $auth_settings = $form_state->getValue('auth_settings');
    $enable_auth = (bool) $form_state->getValue('enable_auth');
    $config->set('enable_auth', $enable_auth);
    $config->set(
      'auth_settings', $enable_auth ? [
        'enable_token_auth' => !empty($auth_settings['enable_token_auth'])
          ? $auth_settings['enable_token_auth'] : FALSE,
        'token_key'         => $auth_settings['token_with_generation']['token_key']
        ?? '',
        'token_user'        => $auth_settings['token_with_generation']['token_user']
        ?? '',
        'enable_basic_auth' => !empty($auth_settings['enable_basic_auth'])
          ? $auth_settings['enable_basic_auth'] : FALSE,
      ] : []
    );

    // Save plugin settings.
    $plugins_values = $form_state->getValue('plugins_wrapper') ?? [];
    foreach ($plugins_values as $plugin_id => $plugin_values) {
      if ($plugin_id === 'tabs') {
        continue;
      }

      // Get the existing plugin instance with stored config:
      $instance = $this->pluginManagerMcp->createInstance($plugin_id);

      // Let it process its form submission.
      $subformState = SubformState::createForSubform(
        $form['plugins_wrapper'][$plugin_id],
        $form,
        $form_state
      );
      $instance->submitConfigurationForm(
        $form['plugins_wrapper'][$plugin_id],
        $subformState
      );

      // Now $instance->configuration is updated.
      // Write it back into our module config array.
      $is_enabled = $instance->getConfiguration()['enabled'] ?? FALSE;
      $config->set("plugins.$plugin_id", [
        'enabled' => $is_enabled,
        'config'  => $is_enabled ? $instance->getConfiguration()['config'] : [],
      ]);
    }

    // Save everything to mcp.settings.
    $config->save();

    // Call parent submit to show success message, etc.
    parent::submitForm($form, $form_state);
  }

}
